/**
 * Par Alain VIZZINI
 * Cration              : 16 mars 2002
 * Dernire modification : 16 mars 2002
 */

using System;
using System.Collections;
using K = System.Console;

using System.IO;
using System.Runtime.Serialization.Formatters.Binary;
using System.Runtime.Serialization;


[Serializable]
public abstract class RendezVous {
    protected DateTime horaire;
    protected String nomPatient;
    private static Hashtable rdvAnnules;

    //accesseurs de horaire
    public DateTime Horaire {
	get {
	    return horaire;
	}
    }

    public abstract int Duree();


    //CONSTRUCTEUR de CLASSE
    static RendezVous(){
      rdvAnnules = new Hashtable();
    }

    /**
     * CONSTRUCTEUR sans paramtre. Initialize le nom  "inconnu" et la date  maintenant
     */
    public RendezVous(){
	nomPatient = "inconnu";
	horaire = DateTime.Now;
    }


    /**
     * CONSTRUCTEUR avec paramtre. Initialise le nom et la date
     */
    public RendezVous(DateTime d, String s){
	horaire = d;
	nomPatient = s;
    }
  

    /**
     * CONSTRUCTEUR avec paramtre. Initialise le nom et la date
     */
    public RendezVous(int an, int mois, int jour, int heure, int min, String s){
	horaire = new DateTime(an,  mois,  jour,  heure,  min, 0);
	nomPatient = s;
    }


    /**
     * Permet de prendre un rdv. Verifie si le rdv en param peut tre ajout dans le SortedList en param et le fait si oui.
     * Preconditions :cetteConsult et consultTree instancis<br> 
     * REMARQUE : le paramtre set est de type IDictionnary qui est une interface qui implmentente la recherche par paire cl-valeur.
     * C'est parce que dans la ralisation j'utilise un SortedList qui a une complexit en O(n) ce qui est relativement minable.
     * En attendant d'implmenter/de trouver une version des arbres binaires de recherche etc... on fera avec !
     * Complexit : O(n)
     */
    protected static bool PrendreRendezVous(RendezVous cetteConsult, IDictionary set){
	bool peutAjouter = true;

	//L je peux affirmer avoir un set instanci Car c'est 
	//fait au chargement de la classe

	if (set.Count == 0){
	    //vide --> ajout automatique
	    set.Add(cetteConsult.Horaire, cetteConsult);
	}
	else {
	    //--> pas vide : peut-on insrer ?
	    //Mthode IDEALE : rcuprer un sous ensemble du set entre [rdv - duree; rdv + duree [
	    //Methode implmente : chouette ! un Iterateur ... :(
	    IDictionaryEnumerator iter = set.GetEnumerator();
      
	    bool arreterRecherche = false;
	    TimeSpan dureeConsult = new TimeSpan(0, cetteConsult.Duree(), 0); //reprsente la duree d'un rdv 

	    //a l'origine, on est plac AVANT le premier element, et ici je sais qu'il y en a un (et oui, set N'EST PAS VIDE !!)
	    while (peutAjouter && iter.MoveNext() && !arreterRecherche){
		//ATTENTION : ne pas utiliser iter.Current qui rends une System.Collections.DictionaryEntry 
		//mais iter.Value qui rend un Object !!
		//En effet, Current est une mthode hrite de IEnumerator et rends ce qui est contenu dans l'element courant. 
		//Or un IDictionnary est compos de doublets cl-valeur !! Il faut donc utiliser Value dfinie 
		//dans IDictionaryEnumerator ...

		peutAjouter = ((iter.Value as RendezVous).Horaire - cetteConsult.Horaire).Duration() > dureeConsult;

		arreterRecherche = ((iter.Value as RendezVous).Horaire - cetteConsult.Horaire) > dureeConsult;
	    }

	    if (peutAjouter)
		set.Add(cetteConsult.Horaire, cetteConsult);
	}
	return peutAjouter;
    }



    public override String ToString(){
	return "Patient " + nomPatient + " : "  + horaire.ToString("f");
    }



    protected static String AnnulerRendezVous(String s, IDictionary set){
	int trouveCombien = 0;
	RendezVous r = null;
      
	if (set.Count == 0){
	    //vide --> ERREUR
	    return "ATTENTION : Le carnet de rendez vous est vide. Requete incongrue.";
	}
	else {
	    //--> pas vide : rdv prsent ??
	    IDictionaryEnumerator i = set.GetEnumerator();
	    while (i.MoveNext()){
		RendezVous tmp = (i.Value as RendezVous);
		if(System.String.Compare(tmp.nomPatient, s, true) == 0){
		    trouveCombien++;
		    r = tmp;
		}
	    }
	}
	if (trouveCombien == 0)
	    return "ERREUR : Pas de rendez vous enregistr pour " + s;
	else
	    if (trouveCombien == 1){
		String msg = "Un rendez vous trouve pour "+ s +" --> " + r;
		int taille = set.Count;
		set.Remove (r.Horaire);
		rdvAnnules.Add (r.Horaire, r);
		if (taille != set.Count)
		    msg = msg + "\n\tEffac avec succs";
		else
		    msg = msg + "\n\t!!! Erreur  la suppression !!!";
		return msg;
	    }
	    else{
		String msg = "ATTENTION : plusieurs rendez vous pour une personne de nom " + s;

		IDictionaryEnumerator i = set.GetEnumerator();
		while (i.MoveNext()){
		    RendezVous tmp = (i.Value as RendezVous);
		    if(System.String.Compare(tmp.nomPatient, s, true) == 0)
			msg = msg + "\n\t--> " + tmp;
		}
		
		return msg;
	    }
    }
    

    protected static String AnnulerRendezVous(RendezVous cetteConsult, IDictionary set){
	if (set.Count == 0){
	    //vide --> ERREUR
	    return "ATTENTION : Le carnet de rendez vous est vide. Requete incongrue.";
	}
	else {
	    //--> pas vide :on supprime
	    if (set.Contains (cetteConsult.Horaire)){
	      //ATTENTION : le set contient effectivement un rdv  la mme heure, mais est-ce pour le mme personne ?
	      //en effet, la recherche (ainsi que l'ordre) est sur la cl mais pas sur le nom.
	      int taille = set.Count;
	      
	      RendezVous rdv = (RendezVous)set[cetteConsult.Horaire];
	      
	      if (System.String.Compare(rdv.nomPatient, cetteConsult.nomPatient, true) == 0){
		set.Remove (cetteConsult.Horaire);
		rdvAnnules.Add(cetteConsult.Horaire, cetteConsult);
		if (taille != set.Count)
		  return "Rendez vous supprim avec succs";
		else
		  return "ERREUR : Impossible de supprimer ce rendez-vous --> " + cetteConsult;
	      }
	      else
		return "ERREUR : Pas de RDV pour " + cetteConsult +"\n\t--> Le RDV est au nom de " + rdv.nomPatient + ".";
	    }
	    else  
	      return "ERREUR : Pas de RDV pour " + cetteConsult;

	}
    }

    /**
     * Sauvegarde les RDV contenus dans set dans le fichier nomFichier. Pour l'instant on sauve en serialisant en binaire.
     * Mais il serait interressant de sauver en XML ...
     * Complexit : O(n)
     * @param nomFichier le nom du fichier (sic)
     * @param set le IDictionary que l'on doit sauvegarder
     * @return true ssi l'opration s'est bien passe
     */
    protected static bool SauvegarderFichier(String nomFichier, IDictionary set){
	try{
	    BinaryFormatter ser = new BinaryFormatter();
	    
	    FileStream fout = new FileStream(nomFichier, FileMode.Create);
	    
	    ser.Serialize(fout, set);
	    
	    fout.Close();
	}
	catch (Exception) { return false; }
	    
      return true;
    }


    /**
     * Lit les RDV contenus dans le fichier nomFichier et les insre dans set. ATTENTION : on remplace le set !!
     *  ATTENTION : ici pas d'abstraction : on parle de SortedSet et pas de IDictionary (pb de cast ...)
     * Complexit : O(n)
     * @param nomFichier le nom du fichier (sic)
     * @param set le SortedList que l'on charge
     * @return true ssi l'opration s'est bien passe
     */
    protected static bool LireFichier(String nomFichier, ref SortedList set){
	    BinaryFormatter ser = new BinaryFormatter();
	    
	    FileStream fout = new FileStream(nomFichier, FileMode.Open);
	    
	    set = (SortedList)ser.Deserialize(fout);
      
	    fout.Close();
	    
	return true;
    }

    /**
     * Lit les RDV contenus dans le fichier nomFichier et les insre dans set. ATTENTION : on remplace le set !!
     *  ATTENTION : ici pas d'abstraction : on parle de Hashtable et pas de IDictionary (pb de cast ...)
     * Complexit : O(n)
     * @param nomFichier le nom du fichier (sic)
     * @param set le SortedList que l'on charge
     * @return true ssi l'opration s'est bien passe
     */
    protected static bool LireFichierAnnules(String nomFichier, ref Hashtable set){
	    BinaryFormatter ser = new BinaryFormatter();
	    
	    FileStream fout = new FileStream(nomFichier, FileMode.Open);
	    
	    set = (Hashtable)ser.Deserialize(fout);
	    
	    fout.Close();
	    
      
      return true;
    }


    /**
     * Affiche sur la sortie std (stdout) la liste des elements d'un TreeSet
     */
    protected static void Lister (IDictionary c) {
	IEnumerable sList = IhmLister (c);
	foreach (String str in sList)
	    K.WriteLine(str);
    }

    /**
     * Retourne dans un tableau de String la reprsentation des RDV d'un TreeSet
     */
    protected static IEnumerable IhmLister (IDictionary c) {
	ArrayList s;

	if (c.Count > 0){
	  s = new ArrayList();
	    
	    IDictionaryEnumerator i = c.GetEnumerator();
	    while (i.MoveNext()){
	      RendezVous tmp = (i.Value as RendezVous);
	      s.Add(tmp.ToString());
	    }
	    
	}
	else{
	    s = new ArrayList(1);
	    s.Add("Vide");
	}
	return s;
    }


    /**
     * Liste sur la sortie standard les RDV annuls.
     */
    public static void ListerAnnules (){
	K.WriteLine("** Les rendez vous annuls");
	Lister (rdvAnnules);
    }


    /**
     * Liste sur la sortie standard les RDV annuls.
     */
    public static IEnumerable IhmListerAnnules (){
	return IhmLister (rdvAnnules);
    }


    /**
     * Recherche tout les RDV dans le service donn (set) et les retourne dans une IEnumerable
     */
    protected static IEnumerable RechercherRendezVous(String s, IDictionary set){
	ArrayList v = new ArrayList();

	if (set.Count != 0){
	    //--> pas vide : rdv prsent ??

	    IDictionaryEnumerator i = set.GetEnumerator();
	    while (i.MoveNext()){
	      RendezVous tmp = (i.Value as RendezVous);
	      if(System.String.Compare(tmp.nomPatient, s, true) == 0)
		  v.Add(tmp.ToString());
	    }
	    
	    
	}
	return v;
    }


    /**
     * A appeler pour sauver le fichier contenant les rdv Annules.<br>
     * <b>ATTENTION</b> : si vous appelez celle de <code>RendezVous</code> vous sauverez les RDV annuls !<br>
     * @return vrai si l'opration s'est bien passe
     */
    public static bool SauverAnnules(){
	return SauvegarderFichier ("rdvAnnules.db", rdvAnnules);

    }

    /**
     * A appeler pour charger le fichier contenant les rdv Annules.<br>
     * <b>ATTENTION</b> : si vous appelez celle de <code>RendezVous</code> vous sauverez les RDV annuls !<br>
     * @return vrai si l'opration s'est bien passe
     */
    public static bool ChargerAnnules(){
	return LireFichierAnnules ("rdvAnnules.db", ref rdvAnnules);

    }

}
