//import RendezVous;
//import RendezVousComparateur;
import java.util.*;

/**
 * Agenda Hospitalier (nom de code : *agandah*)
 * --------------------------------------------
 * @author Julien PUSNIAK
 * @author Alain VIZZINI
 * @version 0.0.1<br>
 * created on 2002-02-05 @ 15h06
 */


public class ConsultationRadiologie extends RendezVous{

    //ATTRIBUTS
    protected static final int duree = 30;    
    private static TreeSet consultationSet = null;

    /*Constructeur PROTECTED pour interdir l'instanciation d'autres classes*/
    protected ConsultationRadiologie(){
	super();

	//pas de constructeur de classe faciles en Java
	if (consultationSet == null)
	    consultationSet = new TreeSet(new RendezVousComparateur());
    }

    private ConsultationRadiologie(int an, int mois, int jour, int heure, int min, String s){
	super(an,  mois,  jour,  heure,  min, s);

	//pas de constructeur de classe faciles en Java
	if (consultationSet == null)
	    consultationSet = new TreeSet(new RendezVousComparateur());      
    }

    public int getDuree(){
	//c'est MINAAAAABLE d'avoir cette mthode en dynamic, mais les statiques Java ne peuvent tre virtuelles
	return duree;
    }




    /**
     * Permet de prendre un rdv EN RADIOLOGIE. Verifie si le rdv en param peut tre ajout dans le TreeSet en param et le fait si oui.<br>
     * <b>Preconditions :</b> cetteConsult et consultTree instancis<br> 
     * @return une chaine rsumant l'opration
     */
    public static String prendreRendezVous(int an, int mois, int jour, int heure, int min, String s) {       
	ConsultationRadiologie r = new ConsultationRadiologie(an, mois, jour, heure, min, s);
	if (prendreRendezVous(r,consultationSet))
	    return "Rendez vous enregistr --> " + r;
	else
	    return "Rendez vous IMPOSSIBLE creneau horaire deja pris pour " + r;
    }




    /**
     * ToString() <br> Retourne une chaine de la forme "RADIOLOGIE : Patient XXX : le MERDREDI 64 AVRIJUIN 2314  25:61"
     * @return comme d'hab
     */
    public String toString(){
	String s = super.toString();
	return "RADIOLOGIE: " + s + ". Dure : " + getDuree() + " minutes.";
    }

    /** 
     * avec ihmToString on ne prcise pas de quel type est le RDV affich (inutile avec le GUI) 
     */
    public String ihmToString(){
	String s = super.toString();
	return s + ". Dure : " + getDuree() + " minutes.";
    }


    /**
     * Annule (retire) le RDV pour la personne de nom s s'il n'y en a qu'un. 
     * Le RDV si annul est plac dans la liste des RDV annuls.<br>
     * Complexit : O(log n)
     * @return un String resumant l'opration
     */
    public static String annulerRendezVous(String s) {       
	return annulerRendezVous(s,consultationSet);
    }

    /**
     * Annule (retire) le RDV pour la personne de nom s  la date donne s'il n'y en a un. 
     * Le RDV si annul est plac dans la liste des RDV annuls.<br>
     * Complexit : O(log n)
     * @return un String resumant l'opration
     */
    public static String annulerRendezVous(int an, int mois, int jour, int heure, int min, String s) {       
	ConsultationRadiologie r = new ConsultationRadiologie(an, mois, jour, heure, min, s);
	return annulerRendezVous(r,consultationSet);
    }

    /**
     * Charge les rdv sur le disque. ATTENTION : on ECRASE l'ancien treeset (penser  svg avant ...)
     * @return true ssi cela s'est bien pass ...
     */
    public static boolean charger(){
	//on ECRASE l'ancien treeset (penser  svg avant ...)
	
	consultationSet = new TreeSet(new RendezVousComparateur());

	return lireFichier ("ConsultationRadiologie.db", consultationSet);
    }

    /**
     * Sauve les rdv sur le disque.
     * @return true ssi cela s'est bien pass ...
     */
    public static boolean sauver(){
	//pas de constructeur de classe faciles en Java
	if (consultationSet != null)
	    return sauvegarderFichier ("ConsultationRadiologie.db", consultationSet);
	else
	    return sauvegarderFichier ("ConsultationRadiologie.db", new TreeSet(new RendezVousComparateur()));
    }


    /**
     * Liste sur la sortie STD les RDV Pediatrie.
     */
    public static void lister () {
	System.out.println("** Les rendez vous Radiologie");
	lister(consultationSet);
    }

    /**
     * Liste dans un tableau de String les RDV Pediatrie.
     */
    public static String [] ihmLister () {
	return ihmLister(consultationSet);
    }


    public static Vector rechercherRendezVous(String s){
	return rechercherRendezVous(s, consultationSet);

    }
}
