import java.io.*;
import java.util.*;
import java.text.*;

/**
 * Agenda Hospitalier (nom de code : *agandah*)
 * --------------------------------------------
 * @author Julien PUSNIAK
 * @author Alain VIZZINI
 * @version 0.0.1<br>
 * created on 2002-02-05 @ 14h32
 */

/**
 * La classe qui implmente les RendezVous. Elle propose les fonctionnalits de base des RendezVous et factorise toutes les mthodes
 * pouvant l'tre.<br>
 * IMPORTANT : c'est cette classe qui gre grce  des attibuts et mthodes statiques les RDV annuls.
 */
public abstract class RendezVous implements Serializable{
    //ATTRIBUTS
    protected GregorianCalendar horaire;
    protected String nomPatient;

    //le TreeSet qui gre les RDV annuls
    private static TreeSet rdvAnnules =null;

    /** 
     * retourne la dure d'un RDV
     * @return la duree du RDV
     */
    public abstract int getDuree();
    
    /**
     * Initialise le TreeSet rdvAnnules car on ne sait pas faire facilement de contructeur
     * de classes en Java
     */
    private static void InitAnnules() {
	if (rdvAnnules == null)
	    rdvAnnules = new TreeSet(new RendezVousComparateur());
    }


    /**
     * CONSTRUCTEUR sans paramtre. Initialize le nom  "inconnu" et la date  maintenant
     */
    public RendezVous(){
	InitAnnules();

	nomPatient = "inconnu";
	horaire = new GregorianCalendar();
	horaire.setGregorianChange(new Date());
    }


    /**
     * CONSTRUCTEUR avec paramtre. Initialise le nom et la date
     */
    public RendezVous(GregorianCalendar d, String s){
	InitAnnules();

	horaire = d;
	nomPatient = s;
    }


    /**
     * CONSTRUCTEUR avec paramtre. Initialise le nom et la date
     */
    public RendezVous(int an, int mois, int jour, int heure, int min, String s){
	InitAnnules();

	horaire = new GregorianCalendar(an,  mois,  jour,  heure,  min);
	nomPatient = s;
    }


    /**
     * Retourne le Gregorian Calendar contenu dans le RendezVous
     * @return le GregorianCalendar contenu dans le RDV
     */
    protected GregorianCalendar getHoraire(){
	return horaire;
    }

    /**
     * Permet de prendre un rdv. Verifie si le rdv en param peut tre ajout dans le TreeSet en param et le fait si oui.<br>
     * <b>Preconditions :</b> cetteConsult et consultTree instancis<br> 
     * @param cetteConsult la consultation  ajouter
     * @param consultTree le TreeSet dans lequel on veut ajouter une consultation
     * @return true ssi le rdv a t ajout au TreeSet
     */
    protected static boolean prendreRendezVous(RendezVous cetteConsult, TreeSet consultTree){
	boolean peutAjouter = true;

	//L je peux affirmer avoir un consultTree instanci Car c'est 
	//fait  la cration de la premire instance de ConsultationXXX

	if (consultTree.isEmpty()){
	    //vide --> ajout automatique
	    consultTree.add(cetteConsult);
	}
	else {
	    //--> pas vide : peut-on insrer ?
	    //Mthode : rcuprer un sous ensemble du set entre [rdv - duree; rdv + duree [

	    //tout d'abord, 2 GregorianCalendar aux bons instants
	    GregorianCalendar gcFrom = new GregorianCalendar();
	    GregorianCalendar gcTo = new GregorianCalendar();
	    gcFrom.setTime (new Date(cetteConsult.getHoraire().getTime().getTime() - cetteConsult.getDuree()*1000*60));
	    gcTo.setTime (new Date(cetteConsult.getHoraire().getTime().getTime() + cetteConsult.getDuree()*1000*60));
	    try {
		// Ensuite, un peu de reflexion masque (comme Zorro) CECI FORCE LES CONSTRUCTEURS 
		// PAR DEFAUT DE ConsultRadio et ConsultPediatrie  tre PROTECTED et non PRIVATE car on les appelle d'une super classe
		// ... vive la rflexion.
		RendezVous rdvFrom = (RendezVous)(cetteConsult.getClass().newInstance());
		rdvFrom.horaire = gcFrom;
		RendezVous rdvTo = (RendezVous)(cetteConsult.getClass().newInstance());
		rdvTo.horaire = gcTo;
	    
		SortedSet tmp = consultTree.subSet(rdvFrom, rdvTo);

		peutAjouter = tmp.size() ==0;
		if (peutAjouter)
		    consultTree.add(cetteConsult);
	    }
	    catch (Exception e) { System.out.println(e); /* Try...Catch oblig pour la petite rflexion ci-dessus */ }
	}
	return peutAjouter;
    }



    /**
     * Permet de savoir si 2 rdv se chevauchent (au niveau du temps)
     * @param cons rdv  comparer avec this
     * @return true ssi le rdv en param et this se chevauchent
     */
    private boolean chevauche(RendezVous cons){
	GregorianCalendar g1;
	GregorianCalendar g2;
	
	g1=getHoraire();
	g2=cons.getHoraire();

	return Math.abs(g1.getTime().getTime()-g2.getTime().getTime())<=getDuree()*60*1000;
    }



    
    /**
     * Annule (retire) le RDV pour la personne de nom s si elle n'en trouve qu'un et propose une liste de RDV possibles si elle en
     * en trouve plusieurs. Le RDV si annul est plac dans la liste des RDV annuls.<br>
     * NB : Non implementee en mode GUI<br>
     * @param s nom du patient dont on veut supprimer le RDV
     * @param consultTree le TreeSet dans lequel on doit chercher un RDV pour s
     * @return un String resumant l'opration
     */
    protected static String annulerRendezVous(String s, TreeSet consultTree){
	int trouveCombien = 0;
	RendezVous r = null;

	if (consultTree.isEmpty()){
	    //vide --> ERREUR
	    return "ATTENTION : Le carnet de rendez vous est vide. Requete incongrue.";
	}
	else {
	    //--> pas vide : rrdv prsent ??
	    Iterator i=consultTree.iterator();
	    while (i.hasNext()){
		RendezVous tmp;
		tmp = (RendezVous)i.next(); 
		if(tmp.getNom().equalsIgnoreCase(s)){
		    trouveCombien++;
		    r = tmp;
		}
	    }
	}
	if (trouveCombien == 0)
	    return "ERREUR : Pas de rendez vous enregistr pour " + s;
	else
	    if (trouveCombien == 1){
		String msg = "Un rendez vous trouve pour "+ s +" --> " + r;
		if (consultTree.remove (r))
		    msg = msg + "\n     Effac avec succs";
		else
		    msg = msg + "\n     !!! Erreur  la suppression !!!";
		return msg;
	    }
	    else{
		String msg = "ATTENTION : plusieurs rendez vous pour une personne de nom " + s;

		Iterator i=consultTree.iterator();
		while (i.hasNext()){
		    RendezVous tmp = (RendezVous)i.next(); 
		    if(tmp.getNom().equalsIgnoreCase(s))
			msg = msg + "\n   --> " + tmp;
		}
		return msg;
	    }
    }


    /**
     * Annule (retire) le RDV identique  cetteConsult (mme date/heure). Le RDV si annul est plac dans la liste des RDV annuls.<br>
     * Complexit : O(log n)
     * @param cetteConsult le RDV  retirer
     * @param consultTree le TreeSet dans lequel on doit chercher un RDV pour s
     * @return un String resumant l'opration
     */
    protected static String annulerRendezVous(RendezVous cetteConsult, TreeSet consultTree){
	if (consultTree.isEmpty()){
	    //vide --> ERREUR
	    return "ATTENTION : Le carnet de rendez vous est vide. Requete incongrue.";
	}
	else {
	    //--> pas vide :on supprime
	    if (consultTree.remove(cetteConsult)){
		rdvAnnules.add(cetteConsult);
		return "Rendez vous supprim avec succs";
	    }
	    else
		return "ERREUR : Impossible de supprimer ce rendez-vous --> " + cetteConsult;
	}
    }


    /**
     * Sauvegarde les RDV contenus dans set dans le fichier nomFichier<br>
     * Complexit : O(n)
     * @param nomFichier le nom du fichier (sic)
     * @param consultTree le TreeSet que l'on doit sauvegarder
     * @return true ssi l'opration s'est bien passe
     */
    protected static boolean sauvegarderFichier(String nomFichier, TreeSet set){
	try {	    
	    FileOutputStream fout;
	    ObjectOutputStream ofout;
	    fout = new FileOutputStream(nomFichier);
	    ofout = new ObjectOutputStream(fout);

	    if (set != null){
		Iterator i=set.iterator();
		while(i.hasNext()){
		    ofout.writeObject(i.next());
		}
	    }
	    ofout.flush();
	    ofout.close();
	}
	catch (OptionalDataException e) {
	    System.out.println("Erreur interne du programme");
	    return false;
	}
	catch (IOException e) { /*genrique IO*/
	    System.out.println("Erreur d'entre/sortie");
	    return false;
	}
	finally {
	}
	return true;
    }




    /**
     * Lit les RDV contenus dans le fichier nomFichier et les insre dans set. ATTENTION : on ne vide pas set, c'est un ajout.<br>
     * PRECOND : set INITIALISE !!<br>
     * Complexit : O(n)
     * @param nomFichier le nom du fichier (sic)
     * @param consultTree le TreeSet que l'on doit sauvegarder
     * @return true ssi l'opration s'est bien passe
     */
    protected static boolean lireFichier(String nomFichier, TreeSet set){
	try {	    
	    FileInputStream fin;
	    ObjectInputStream ofin;
	    fin = new FileInputStream(nomFichier);
	    ofin = new ObjectInputStream(fin);

	    if (set != null){
		RendezVous r;
		while ((r = (RendezVous)ofin.readObject()) != null){
		    set.add (r);
		}
	    }
	    //pas de else : on ne peut crer de set car on passe les params par valeur 
	    //et par consquent le set cr serait ignor hors de cette methode ...

	    ofin.close();
	}
	catch (OptionalDataException e) {
	    System.out.println("Erreur interne du programme : OptionalDataException");
	    return false;
	}
	catch (ClassNotFoundException e) {
	    System.out.println("Erreur interne du programme : ClassNotFoundException");
	    return false;
	}
	catch (FileNotFoundException e) {
	    System.out.println("Erreur interne du programme : FileNotFound");
	    return false;
	}
	catch (EOFException e) { /* fin de fichier : c ok*/
	    return true;
	}
	catch (IOException e) { /*genrique IO*/
	    System.out.println("Erreur d'entre/sortie : "  + e);
	    return false;
	}
	finally {
	}

	return true;
    }


    
    /**
     * ToString() <br> Retourne une chaine de la forme "Patient XXX : le MERDREDI 64 AVRIJUIN 2314  25:61"
     * @return comme d'hab
     */
    public String toString(){
	Locale.setDefault (Locale.FRANCE);
	SimpleDateFormat sdf = new SimpleDateFormat ("'le' EEEE dd MMM yyyy '' kk:mm");
	String s = "Patient " + nomPatient + " : "  +sdf.format(horaire.getTime());

	return s;
    }

    /** 
     * avec ihmToString on ne prcise pas de quel type est le RDV affich (inutile avec le GUI) 
     */
    public abstract String ihmToString();
 


    /** 
     * devinez ... allez, c'est le nom du patient du RDV.
     */
    public String getNom(){
	return nomPatient;
    }

    /**
     * A appeler pour charger le fichier contenant les rdv.<br>
     * <b>ATTENTION</b> : si vous appelez celle de <code>RendezVous</code> vous chargerez les RDV annuls !<br>
     * @return vrai si l'opration s'est bien passe
     */
    public static boolean charger(){
	//on ECRASE l'ancien treeset (penser  svg avant ...)

	rdvAnnules = new TreeSet(new RendezVousComparateur());

	return lireFichier ("rdvAnnules.db", rdvAnnules);
    }

    /**
     * A appeler pour sauver le fichier contenant les rdv.<br>
     * <b>ATTENTION</b> : si vous appelez celle de <code>RendezVous</code> vous sauverez les RDV annuls !<br>
     * @return vrai si l'opration s'est bien passe
     */
    public static boolean sauver(){
	//pas de constructeur de classe faciles en Java
	if (rdvAnnules != null)
	    return sauvegarderFichier ("rdvAnnules.db", rdvAnnules);
	else
	    return sauvegarderFichier ("rdvAnnules.db", new TreeSet(new RendezVousComparateur()));
    }




    /**
     * Affiche sur la sortie std (stdout) la liste des elements d'un TreeSet
     */
    protected static void lister (TreeSet c) {
	String [] s = ihmLister (c);
	for (int i=0; i< s.length; i++)
	    System.out.println(s[i]);
    }
    /**
     * Retourne dans un tableau de String la reprsentation des RDV d'un TreeSet
     */
    protected static String[] ihmLister (TreeSet c) {
	String [] s;

	if (c != null && c.size() > 0){
	    s = new String [c.size()];
	    int idx = 0;
	    Iterator i = c.iterator();

	    while(i.hasNext())
		s[idx++] = ((RendezVous)i.next()).ihmToString();
	}
	else{
	    s = new String[1];
	    s[0] = "Vide";
	}
	return s;
    }

    /**
     * Liste sur a sortie standard les RDV annuls.
     */
    public static void listerAnnules (){
	System.out.println("** Les rendez vous annuls");
	lister (rdvAnnules);
    }

    /**
     * REtourne un tableau de String contenant toute les entres des RDV annuls.
     */
    public static String [] ihmListerAnnules (){
	return ihmLister (rdvAnnules);
    }




    protected static Vector rechercherRendezVous(String s, TreeSet consultTree){
	int trouveCombien = 0;
	Vector v = new Vector();

	if (!consultTree.isEmpty()){
	    //--> pas vide : rdv prsent ??
	    Iterator i=consultTree.iterator();
	    while (i.hasNext()){
		RendezVous tmp;
		tmp = (RendezVous)i.next(); 
		if(tmp.getNom().equalsIgnoreCase(s)){
		    trouveCombien++;
		    v.add (tmp.toString());
		}
	    }
	}
	return v;
    }

}
